@@ -65,7 +65,7 @@ rep_blank_na = function(dt) {
 }
 
 # check y
-check_y = function(dt, y, positive) {
+check_y = function(dt, y, positive="bad|1") {
   dt = setDT(dt)
   positive = as.character(positive)
   # dt[[y]]  = as.character(dt[[y]])
@@ -118,13 +118,13 @@ check_print_step = function(print_step) {
 }
 
 # x variable
-x_variable = function(dt, y, x) {
-  x_all = setdiff(names(dt), y)
+x_variable = function(dt, y, x, var_skip=NULL) {
+  x_all = setdiff(names(dt), c(y, var_skip))
 
   if (is.null(x)) x = x_all
 
   if ( length(setdiff(x,x_all)) > 0 ) {
-    warning(sprintf('Incorrect inputs; there are %s variables are not exist in the input dataframe, which are removed from x. \n%s', length(setdiff(x, x_all)), paste(setdiff(x, x_all), collapse = ', ')) )
+    warning(sprintf('Incorrect inputs; there are %s variables are not exist in the input data frame, which are removed from x. \n%s', length(setdiff(x, x_all)), paste(setdiff(x, x_all), collapse = ', ')) )
     x = intersect(x, x_all)
   }
 
